# Great Deep Learning Tutorials for Compter Vision
A Great Collection of Deep Learning Tutorials and Repositories for Compter Vision

## General:
- [Microsoft Computer Vision Recipes](https://github.com/microsoft/computervision-recipes) [_Great_]  
- [OpenMMLab](https://openmmlab.com/) [_Great_]  
- [OpenMMLab - GitHub](https://github.com/open-mmlab)  
- [OpenMMLab - MMCV is a foundational python library for computer vision research](https://github.com/open-mmlab/mmcv)  
- [OpenMMLab - MMEditing is an open source image and video editing toolbox](https://github.com/open-mmlab/mmediting)  
- [OpenMMLab - MMDetection](https://github.com/open-mmlab/mmdetection)  
- [Kornia - is a differentiable computer vision library for PyTorch](https://github.com/kornia/kornia)    

## Model Visualization:
- [CNN Visualizations](https://github.com/utkuozbulak/pytorch-cnn-visualizations) [_Very Good_]  
- [CNN-heatmap](https://github.com/mrgloom/CNN-heatmap)  
- [Tools to Design or Visualize Architecture of Neural Network](https://github.com/ashishpatel26/Tools-to-Design-or-Visualize-Architecture-of-Neural-Network) [**Great**]  
- [Netron - GitHub](https://github.com/lutzroeder/netron) [_Excellent_]  
- [Monitor your GPUs](https://github.com/msalvaris/gpu_monitor) [**Excellent**]  
- [Understanding CNN](http://cs231n.github.io/understanding-cnn/)  
- [Exploring Neural Networks with Activation Atlases](https://distill.pub/2019/activation-atlas/)  [_Great_]
- [Explaining What Explainable AI Did Not](https://bair.berkeley.edu/blog/2020/04/23/decisions/) [_Interesting_]
- [CNN Explainer](https://poloclub.github.io/cnn-explainer/) [_Interesting_]  
- [Interactive Tools for ML, DL and Math](https://github.com/Machine-Learning-Tokyo/Interactive_Tools#embedding-projector)  [_Interesting_]  
- [Visualizing Neural Networks with the Grand Tour](https://distill.pub/2020/grand-tour/) [_Interesting_]    
- [Zoom In: An Introduction to Circuits](https://distill.pub/2020/circuits/zoom-in/)  
- [Concept: Concept Modeling on Images](https://github.com/MaartenGr/Concept)  

### GradCAM:
- [GradCAM Main Paper](https://arxiv.org/abs/1610.02391)  
- [PyTorch GradCAM 1](https://github.com/jacobgil/pytorch-grad-cam)  
- [PyTorch GradCAM 2](https://github.com/kazuto1011/grad-cam-pytorch)  
- [Keras GradCAM](https://keras.io/examples/vision/grad_cam/)  
- [pyimagesearch GradCAM](https://www.pyimagesearch.com/2020/03/09/grad-cam-visualize-class-activation-maps-with-keras-tensorflow-and-deep-learning/)  

### t-SNE Visualization:
- [TSNE-Visualization of large dataset images using pre-trained networks in Tensorflow and Keras](https://e3oroush.github.io/tsne-visualization/)  [_Great_] 

### UMAP Visualization:
- [Understanding UMAP](https://pair-code.github.io/understanding-umap/)

### EDA and visualization of Image/Video Datasets:
- [fastdup: a powerful free tool designed to rapidly extract valuable insights from your image & video datasets](https://github.com/visual-layer/fastdup)  

## Image Classification Models:
- [GhostNet (CVPR 2020) in PyTorch and TensorFlow](https://blog.paperspace.com/ghostnet-cvpr-2020/)  
- [GhostNet - GitHub](https://github.com/huawei-noah/ghostnet)  
- [GhostNet - PyTorch Hub](https://pytorch.org/hub/pytorch_vision_ghostnet/) [_Excellent_]   
- [Residual blocks — Building blocks of ResNet](https://towardsdatascience.com/residual-blocks-building-blocks-of-resnet-fd90ca15d6ec)  
- [EfficientNet-PyTorch](https://github.com/lukemelas/EfficientNet-PyTorch)  
- [EfficientNet Explanation](https://amaarora.github.io/2020/08/13/efficientnet.html)  
- [DeepMind - NFNets](https://github.com/deepmind/deepmind-research/tree/master/nfnets)  
- [NFNets - PyTorch](https://github.com/vballoli/nfnets-pytorch)  
- [EfficientNetV2](https://arxiv.org/pdf/2104.00298.pdf)  
- [Deit: Data-Efficient architectures and training for Image classification](https://github.com/facebookresearch/deit)  
- [How to Train State-Of-The-Art Models Using TorchVision’s Latest Primitives](https://pytorch.org/blog/how-to-train-state-of-the-art-models-using-torchvision-latest-primitives/) [_Excellent_]  

### Data Augmentation for Image Classification:
- [Image Augmentation](https://github.com/aleju/imgaug)  
- [AugLy - data augmentations library that supports different modalities](https://github.com/facebookresearch/AugLy)  
- [Learnable Test-time Augmentation](https://github.com/bayesgroup/gps-augment)  

## Self-Supervised Learning:
- [VISSL - VIsion library for state-of-the-art Self-Supervised Learning](https://vissl.ai/)  
- [VISSL - GitHub](https://github.com/facebookresearch/vissl)  
- [DINO: Self-Supervised Vision Transformers](https://github.com/facebookresearch/dino)  
- [DINOv2: Learning Robust Visual Features without Supervision](https://github.com/facebookresearch/dinov2)
- [I-JEPA (the Image-based Joint-Embedding Predictive Architecture)](https://github.com/facebookresearch/ijepa)  

## Transformers in Computer Vision:
- [Vision Transformers Tutorial](https://blog.paperspace.com/vision-transformers/) [**Great**]   
- [Transformers in computer vision: ViT architectures, tips, tricks and improvements](https://theaisummer.com/transformers-computer-vision/) [Great]  
- [Vision Transformer](https://github.com/google-research/vision_transformer)  
- [Vision Transformer - Pytorch](https://github.com/lucidrains/vit-pytorch) [**Great**]   
- [PyTorch External Attention](https://github.com/xmu-xiaoma666/External-Attention-pytorch) [**Great**]   
- [MobileViT in PyTorch](https://github.com/chinhsuanwu/mobilevit-pytorch)    
- [Clip-vit](https://huggingface.co/openai/clip-vit)  
- [OpenAI CLIP](https://github.com/openai/CLIP)  
- [Transformers: from NLP to CV](https://github.com/IbrahimSobh/Transformers) [**Very Great & Excellent**]   

### Vision-Language Models (LLMs in Image & Computer Vision):
- [Prismer: A Vision-Language Model](https://github.com/NVlabs/prismer)  
- [ViperGPT: Visual Inference via Python Execution for Reasoning](https://github.com/cvlab-columbia/viper)  
- [LLaVA: Large Language and Vision Assistant](https://llava-vl.github.io/)  
- [ImageBind-LLM](https://github.com/OpenGVLab/LLaMA-Adapter/tree/main/imagebind_LLM)  

### Multi-Modal LLMs:
- [CoDi: Any-to-Any Generation via Composable Diffusion](https://github.com/microsoft/i-Code/tree/main/i-Code-V3)
- [kosmos-2: Grounding Multimodal Large Language Models to the World](https://github.com/microsoft/unilm/tree/master/kosmos-2)  

### Language-Vision Intelligence:
- [Salesforce LAVIS: A Library for Language-Vision Intelligence](https://github.com/salesforce/LAVIS)  

## Generative AI in Image and Computer Vision:
- [Dream Gaussian: Generative Gaussian Splatting for Efficient 3D Content Creation](https://github.com/dreamgaussian/dreamgaussian)
- [Stable Diffusion Video](https://www.linkedin.com/posts/eric-vyacheslav-156273169_stability-just-released-stable-diffusion-activity-7133533117700521984-JWpc?utm_source=share&utm_medium=member_desktop)
- [Tutorial on Diffusion Models for Imaging and Vision](https://arxiv.org/abs/2403.18103)  

## Detection & Segmentation:
## Object Detection:
- [Object Detection for Dummies Part 1](https://lilianweng.github.io/lil-log/2017/10/29/object-recognition-for-dummies-part-1.html)  
- [Object Detection for Dummies Part 2](https://lilianweng.github.io/lil-log/2017/12/15/object-recognition-for-dummies-part-2.html)  
- [Object Detection for Dummies Part 3](https://lilianweng.github.io/lil-log/2017/12/31/object-recognition-for-dummies-part-3.html)  
- [Object Detection for Dummies Part 4](https://lilianweng.github.io/lil-log/2018/12/27/object-detection-part-4.html)  
- [Open MMLab Detection Toolbox and Benchmark](https://github.com/open-mmlab/mmdetection)  
- [RetinaNet: how Focal Loss fixes Single-Shot Detection](https://towardsdatascience.com/retinanet-how-focal-loss-fixes-single-shot-detection-cb320e3bb0de)  
- [Getting Started With Bounding Box Regression In TensorFlow](https://towardsdatascience.com/getting-started-with-bounding-box-regression-in-tensorflow-743e22d0ccb3)  
- [Pelee: A Real-Time Object Detection System on Mobile Devices](https://github.com/Robert-JunWang/Pelee)  
- [Pelee: Tutorial](https://medium.com/@bibekchaudhary/pelee-real-time-object-detection-system-on-mobile-devices-f565947c04c4)  
- [An overview of deep-learning based object-detection algorithms](https://medium.com/@fractaldle/brief-overview-on-object-detection-algorithms-ec516929be93)  
- [Object detection and tracking in PyTorch](https://towardsdatascience.com/object-detection-and-tracking-in-pytorch-b3cf1a696a98)  
- [Object Detection with RetinaNet - Keras](https://keras.io/examples/vision/retinanet/)  
- [PP-YOLO](https://github.com/PaddlePaddle/PaddleDetection/blob/release/0.4/configs/ppyolo/README.md)  
- [Super Fast and Accurate 3D Object Detection based on 3D LiDAR Point Clouds](https://github.com/maudzung/Super-Fast-Accurate-3D-Object-Detection)  
- [Signfeld - Synthetic traffic sign detection](https://github.com/moabitcoin/signfeld)  
- [Swin-Transformer](https://github.com/microsoft/Swin-Transformer)  
- [DETR - DEtection TRansformer](https://github.com/facebookresearch/detr) [_Great_]  
- [YOLOX](https://github.com/Megvii-BaseDetection/YOLOX)   
- [Yolo-v5 vs EfficientDet](https://blog.roboflow.com/yolov5-is-here/)  
- [MMRotate: OpenMMLab Rotated Object Detection](https://github.com/open-mmlab/mmrotate) [**Great**]  
- [YOLO v7](https://github.com/WongKinYiu/yolov7)  
- [Ultralytics YOLOv8](https://github.com/ultralytics/ultralytics)  
- [Yolov8 Tracking](https://github.com/mikel-brostrom/yolov8_tracking)  
- [YOLO-NAS](https://github.com/Deci-AI/super-gradients/blob/master/YOLONAS.md)  

### Augmentation for Object Detection & Instance Segmentation:
- [Albumentations - Great Library for image augmentation and transformations](https://github.com/albumentations-team/albumentations)    
- [Shapely - Manipulation and analysis of geometric objects](https://github.com/Toblerity/Shapely)  

### Faster R-CNN Object Detector Tutorials:
- [Guide to build Faster RCNN in PyTorch](https://medium.com/@fractaldle/guide-to-build-faster-rcnn-in-pytorch-95b10c273439)  
- [Simple and Fast Implementation of Faster R-CNN](https://github.com/chenyuntc/simple-faster-rcnn-pytorch)  
- [PyTorch Tutorial to Object Detection](https://github.com/sgrvinod/a-PyTorch-Tutorial-to-Object-Detection)  
- [Object Detection and Classification using R-CNNs](http://www.telesens.co/2018/03/11/object-detection-and-classification-using-r-cnns/)  
- [Faster R-CNN: Down the rabbit hole of modern object detection](https://tryolabs.com/blog/2018/01/18/faster-r-cnn-down-the-rabbit-hole-of-modern-object-detection/)  
- [Faster R-CNN (object detection) implemented by Keras for custom data](https://towardsdatascience.com/faster-r-cnn-object-detection-implemented-by-keras-for-custom-data-from-googles-open-images-125f62b9141a)  
- [Mask R-CNN Unmasked](https://medium.com/@fractaldle/mask-r-cnn-unmasked-c029aa2f1296)  

### Instance Segmentation:  
- [Image Segmentation: tips and tricks from 39 Kaggle competitions](https://neptune.ai/blog/image-segmentation-tips-and-tricks-from-kaggle-competitions?utm_source=reddit&utm_medium=post&utm_campaign=blog-image-segmentation-tips-and-tricks-from-kaggle-competitions) [_Excellent_]  

### Semantic Segmentation:
- [Simple Guide to Semantic Segmentation](https://medium.com/beyondminds/a-simple-guide-to-semantic-segmentation-effcf83e7e54)  
- [Background Matting](https://github.com/PeterL1n/BackgroundMattingV2)    
- [Facebook's Segment Anything Model (SAM)](https://github.com/facebookresearch/segment-anything)  

### License Plate Detection and Recognition:
- [ALPR in Unscontrained Scenarios](https://github.com/sergiomsilva/alpr-unconstrained) [_Good_]  
- [ALPR in Unscontrained Scenarios - Project Page](http://sergiomsilva.com/pubs/alpr-unconstrained/)  
- [Lisence plate detection and recognition - ANPR](https://github.com/GuiltyNeuron/ANPR)  
- [License Plate Detection](https://github.com/AzharMithani/License-Plate-Detection)  
- [yolo3 to detection license plate](https://github.com/ThorPham/License-plate-detection)  

### Keypoint Regression:
- [Transfer Learning Library](https://github.com/thuml/Transfer-Learning-Library)  
- [DEKR: dense keypoint regression framework](https://github.com/HRNet/DEKR)  
- [CenterNet](https://github.com/xingyizhou/CenterNet)  
- [AdelaiDet](https://github.com/aim-uofa/AdelaiDet)  
- [Keypoint Regression](https://github.com/Srijan2001/Keypoint-regression)  

## Loss Functions:
- [Use Focal Loss To Train Model Using Imbalanced Dataset](https://leimao.github.io/blog/Focal-Loss-Explained/)  

## Image Augmentation:
- [Image Augmentation for more train data - Kaggle](https://www.kaggle.com/vishnurapps/image-augmentation-for-more-training-data) [_Great_]  

## Deformable Convolution PyTorch Implementation:
- [TorchVision - Deformable Convolution - Link1](https://github.com/pytorch/vision/releases)  
- [TorchVision - Deformable Convolution - Link2](https://github.com/pytorch/vision/blob/master/torchvision/ops/deform_conv.py)  
- [Simple PyTorch Deformable Convolution v2](https://github.com/developer0hye/Simple-PyTorch-Deformable-Convolution-v2) [Great]  
- [Deformable Convolutional Networks v2 with Pytorch](https://github.com/CharlesShang/DCNv2/tree/pytorch_1.0)  [_Good_]  
- [Deformable ConvNets v2](https://github.com/4uiiurz1/pytorch-deform-conv-v2)  
- [MMLab Detection Toolbox](https://github.com/open-mmlab/mmdetection/tree/master/mmdet/ops/dcn)  
- [DCNv2 in PyTorch](https://github.com/chengdazhi/Deformable-Convolution-V2-PyTorch)  
- [PyTorch implementation of Deformable Convolution](https://github.com/oeway/pytorch-deform-conv)  


## 3D Image Classification & 3D Computer Vision:
- [3D MNIST](https://medium.com/shashwats-blog/3d-mnist-b922a3d07334)  
- [3D CNN Images with Tensorflow](https://github.com/jibikbam/CNN-3D-images-Tensorflow)  
- [Point Cloud Library (PCL)](https://github.com/PointCloudLibrary/pcl)  
- [Kaolin - a PyTorch library for accelerating 3D deep learning](https://github.com/NVIDIAGameWorks/kaolin)  
- [PyTorch3D](https://github.com/facebookresearch/pytorch3d) [_Fantastic_]  
- [OpenCV Tutorial Homography](https://docs.opencv.org/master/d9/dab/tutorial_homography.html)  
- [Total 3D Understanding](https://github.com/yinyunie/Total3DUnderstanding)   

## OCR:
- [Easy OCR](https://github.com/JaidedAI/EasyOCR) [**Great**]  
- [MMOCR](https://github.com/open-mmlab/mmocr) [**Great**]
- [Word Level OCR Dataset for Persian Language](https://github.com/amirabbasasadi/Shotor)  
- [Simple Persian Word-Level OCR](https://github.com/amirabbasasadi/PersianOCR)  
- [Amazon Textract - OCR](https://aws.amazon.com/textract/)  
- [TextFuseNet](https://github.com/ying09/TextFuseNet)  
- [Transformer-OCR](https://github.com/fengxinjie/Transformer-OCR)  
- [Microsoft TrOCR](https://github.com/microsoft/unilm/tree/master/trocr)  
- [vedastr: open source scene text recognition toolbox](https://github.com/Media-Smart/vedastr)  
- [Goodbye OCR - Welcome Donut from MIT](https://www.linkedin.com/posts/aagarwal29_sagemaker-donut-mit-activity-7067002244352651265-yvfo/?utm_source=share&utm_medium=member_android)
- [Nougat: Neural Optical Understanding for Academic Documents](https://github.com/facebookresearch/nougat)
- [Accurate line-level text detection and recognition (OCR) in any language](https://github.com/VikParuchuri/surya)  

### Persian OCR:  
- [Persian Dataset](https://github.com/persiandataset)  
- [Arshasb: Persian OCR dataset](https://github.com/persiandataset/Arshasb)   
- [Above Data Set (complete set)](https://drive.google.com/file/d/1Ahw_y-ugnu6FFz_SRBm2VUxnaqmXfsvp)  

## Few-Shot Learning:
- [Advances in few-shot learning](https://towardsdatascience.com/advances-in-few-shot-learning-a-guided-tour-36bc10a68b77)  
- [Advances in few-shot learning: reproducing results in PyTorch](https://towardsdatascience.com/advances-in-few-shot-learning-reproducing-results-in-pytorch-aba70dee541d)  
- [One Shot learning, Siamese networks and Triplet Loss](https://medium.com/@crimy/one-shot-learning-siamese-networks-and-triplet-loss-with-keras-2885ed022352)  
- [Meta-Learning with Differentiable Convex Optimization](https://github.com/kjunelee/MetaOptNet)  
- [Building a One-Shot Learning Network with PyTorch](https://towardsdatascience.com/building-a-one-shot-learning-network-with-pytorch-d1c3a5fafa4a)  
- [Annotation-Efficient Learning](https://annotation-efficient-learning.github.io/) [_Good Few-Shot Learning Tutorial_]  
- [Awesome Papers - Few shot](https://github.com/Duan-JM/awesome-papers-fewshot)  
- [Few-Shot-Learning](https://github.com/daooshee/Few-Shot-Learning)  
- [OpenMMLab FewShot Learning](https://github.com/open-mmlab/mmfewshot) [Great]     

## Learning to Hash & General Hashing (+ Fast Searching Methods):
- [Finding similar images using Deep learning and Locality Sensitive Hashing](https://towardsdatascience.com/finding-similar-images-using-deep-learning-and-locality-sensitive-hashing-9528afee02f5) [_Very Good_]  
- [Image similarity using Triplet Loss](https://towardsdatascience.com/image-similarity-using-triplet-loss-3744c0f67973)  
- [Finding duplicate images made easy!](https://github.com/idealo/imagededup)  
- [Duplicate Image Detection - perspective hash (pHash)](https://github.com/philipbl/duplicate-images)  
- [ImageHash](https://github.com/JohannesBuchner/imagehash)  
- [NGT - Neighborhood Graph and Tree for Indexing High-dimensional Data](https://github.com/yahoojapan/NGT) [_Great_]  
- [NGT - Tutorial](https://github.com/yahoojapan/NGT/wiki)  
- [NGT - Python](https://github.com/yahoojapan/NGT/blob/master/python/README.md)  
- [Visual Search with MXNet Gluon and HNSW](https://github.com/ThomasDelteil/VisualSearch_MXNet)  
- [Annoy: Approximate Nearest Neighbors in C++/Python](https://github.com/spotify/annoy) [**Great**]     
- [datasketch: Big Data Looks Small](https://github.com/ekzhu/datasketch) [**Great:** probabilistic data structures that can process and search very large amount of data super fast]   
- [Image Similarity with Hugging Face Datasets and Transformers](https://huggingface.co/blog/image-similarity)  

## Video Understanding: 
- [Holistic Video Understanding Challenge](https://holistic-video-understanding.github.io/workshops/iccv2019.html)  
- [Holistic Video Understanding Dataset](https://github.com/holistic-video-understanding/Mini-HVU)  
- [Holistic Large Scale Video Understanding - Tutorial](https://towardsdatascience.com/holistic-large-scale-video-understanding-c423701b777a)  
- [Deep Learning on Video - Part1](https://towardsdatascience.com/deep-learning-on-video-part-one-the-early-days-8a3632ed47d4)   
- [VMZ: Model Zoo for Video Modeling](https://github.com/facebookresearch/VMZ)  
- [The 3rd YouTube-8M Video Understanding Challenge - 1st Place Solution](https://www.kaggle.com/c/youtube8m-2019/discussion/112869#latest-668179?utm_medium=email&utm_source=intercom&utm_campaign=competition-recaps-yt-2019)  
- [torchvideo](https://torchvideo.readthedocs.io/en/latest/#)  
- [torchvideo - GitHub](https://github.com/torchvideo/torchvideo)  
- [PytorchVideo - GitHub](https://github.com/facebookresearch/pytorchvideo) [_Great_]  
- [PytorchVideo - main page](https://pytorchvideo.org/) [_Great_]  
- [MoViNet-pytorch](https://github.com/Atze00/MoViNet-pytorch) [Interesting]  
- [Basic Video transforms for PyTorch](https://github.com/hassony2/torch_videovision)  
- [Fast and Easy to use video feature extractor](https://github.com/antoine77340/video_feature_extractor)  
- [Video Augmentation Techniques for Deep Learning](https://github.com/okankop/vidaug) [_Great_]  
- [Decord_loader - Excellent Video Data Loader](https://gluon-cv.mxnet.io/build/examples_action_recognition/decord_loader.html) [_Great_]    
- [Decord - GitHub](https://github.com/dmlc/decord)  
- [PyAV - Pythonic binding for the FFmpeg libraries](https://github.com/PyAV-Org/PyAV)  
- [Python bindings for FFmpeg](https://github.com/kkroening/ffmpeg-python)  
- [GStreamer: Multimedia Framework](https://github.com/GStreamer/gstreamer)   
- [NÜWA: text to video synthesis](https://github.com/lucidrains/nuwa-pytorch)  
- [Awesome Video Datasets](https://github.com/xiaobai1217/Awesome-Video-Datasets)  
- [Iranian Movies Kaggle Dataset](https://www.kaggle.com/datasets/arianghasemi/iranian-movies)  
- [Kinetics 400 Data Set - Download Link via DropBox](https://www.dropbox.com/s/wcs01mlqdgtq4gn/compress.tar.gz?dl=1)  


## Text-to-Video: 
- [Text to Video Synthesis](https://huggingface.co/spaces/damo-vilab/modelscope-text-to-video-synthesis)  

## Optical Flow: 
- [RAFT](https://github.com/princeton-vl/RAFT)  


## Visual Attention Method:
- [Attention Mechanisms in Computer Vision Part 1: CBAM](https://blog.paperspace.com/attention-mechanisms-in-computer-vision-cbam/) [_Excellent_]  
- [Efficient Channel Attention for Deep Convolutional Neural Networks (ECA-Net)](https://blog.paperspace.com/attention-mechanisms-in-computer-vision-ecanet/)  
- [ECA-Net: Efficient Channel Attention](https://github.com/BangguWu/ECANet)  
- [Channel Attention and Squeeze-and-Excitation Networks (SENet)](https://blog.paperspace.com/channel-attention-squeeze-and-excitation-networks/)  
- [Self-Attention In Computer Vision](https://towardsdatascience.com/self-attention-in-computer-vision-2782727021f6)  


## Visual Question Answering (VQA):
- [Vanilla VQA](https://medium.com/ai2-blog/vanilla-vqa-adcaaaa94336)  


## Pose Estimation:
- [Pose Animator](https://github.com/yemount/pose-animator/)  
- [RepNet: Weakly Supervised 3D Human Pose Estimation](https://github.com/bastianwandt/RepNet)  


## Object Tracking:
- [GSDT](https://github.com/yongxinw/GSDT)  


## GANs:
- [MMGeneration](https://github.com/open-mmlab/mmgeneration)  
- [Generate Anime Style Face Using DCGAN and Explore Its Latent Feature Representation](https://towardsdatascience.com/generate-anime-style-face-using-dcgan-and-explore-its-latent-feature-representation-ae0e905f3974)  
- [StyleGAN2](https://github.com/EvgenyKashin/stylegan2-distillation)  
- [DeepFaceLab](https://github.com/iperov/DeepFaceLab)   
- [imagen-pytorch: Google's Text-to-Image Neural Network](https://github.com/lucidrains/imagen-pytorch) [Great]  
- [DragGAN](https://github.com/XingangPan/DragGAN)  

## Image Super-Resolution:
- [Image Super-Resolution: A Comprehensive Review](https://blog.paperspace.com/image-super-resolution/)  


## Siamese Networks:
- [Siamese Network Tutorial with TensorFlow](https://leimao.github.io/article/Siamese-Network-MNIST/)  


## Model Evaluation:
- [Precision and Recall for Multi-Class](https://stats.stackexchange.com/questions/51296/how-do-you-calculate-precision-and-recall-for-multiclass-classification-using-co)  
- [Precision and Recall](https://en.wikipedia.org/wiki/Precision_and_recall)  
- [Precision-Recall metric](https://scikit-learn.org/stable/auto_examples/model_selection/plot_precision_recall.html)  


## Model Size & Speed of Models:
- [GFLOPs & Number of Parameters of ResNet Models](https://neurohive.io/wp-content/uploads/2019/01/resnet-architectures-34-101.png)  
- [EfficientNet Models Accuracy vs FLOPs](https://raw.githubusercontent.com/tensorflow/tpu/master/models/official/efficientnet/g3doc/flops.png)  


## Computer Vision Annotation Tools:
- [CVAT](https://github.com/opencv/cvat)  
- [SuperAnnotate](https://www.superannotate.com/)  
- [OpenCV - SuperAnnotate Desktop](https://opencv.org/superannotate-desktop/)  
- [Curve-GCN](https://github.com/fidler-lab/curve-gcn)  
- [VoTT (Visual Object Tagging Tool)](https://github.com/microsoft/VoTT)  
- [MakeSense AI](https://www.makesense.ai/)  
- [Awesome Data Labeling](https://github.com/heartexlabs/awesome-data-labeling)  

## Building Datasets:
- [Building Image Datasets for Computer Vision Algorithms](https://blog.paperspace.com/building-computer-vision-datasets/)  
- [icrawler - mini image framework of web crawlers](https://github.com/hellock/icrawler) [_Great_]   
- [Unity Computer Vision](https://github.com/Unity-Technologies/com.unity.perception)  
- [pytube - downloading YouTube Videos](https://github.com/pytube/pytube)  
- [Subreddit Media Downloader](https://github.com/Psycoguana/SubredditMediaDownloader)  
- [pigeon: Quickly annotate data on Jupyter](https://github.com/agermanidis/pigeon)  

## LLMs & Generative AI in Vision Tasks:
- [Draw UI via Generative AI](https://www.linkedin.com/posts/eric-vyacheslav-156273169_this-will-change-the-whole-industry-draw-a-ui-activity-7130527961169969153-JAi5?utm_source=share&utm_medium=member_desktop)
- [Emu video & Emu edit meta's Models](https://www.linkedin.com/posts/aiatmeta_today-were-announcing-two-new-milestones-activity-7130948419409375232-7sAh?utm_source=share&utm_medium=member_desktop)  

## Other:

### Color Channel Order:
- OpenCV is **BGR**, Pillow is **RGB**, and Decord is **RGB**  
  - [Numpy Image bgr-to-rgb](https://www.scivision.dev/numpy-image-bgr-to-rgb/)   
  
- [Imagenet 1000 class indices to human readable labels](https://gist.github.com/yrevar/942d3a0ac09ec9e5eb3a)  

- [streamlit Make Images & Faces as Comic one!!](https://share.streamlit.io/nathannguyen-dev/comic_me_v1/main.py)   
- [How DALL-E 2 could solve major computer vision challenges](https://venturebeat.com/2022/04/16/how-dall-e-2-could-solve-major-computer-vision-challenges/)  
- [PyTorch jpeg Decoding on the GPU](https://pytorch.org/blog/pytorch-1.9-new-library-releases/#beta-jpeg-decoding-on-the-gpu)  
