# Great Deep Learning Tutorials for Data Science
A Great Collection of Deep Learning Tutorials and Repositories for Data Science  

## General:
- [Python Data Science Tutorials](https://realpython.com/tutorials/data-science/)  
- [Data Science Course](https://github.com/amingheibi/Data-Science-Course)  
- [TensorFlow Decision Forests](https://github.com/tensorflow/decision-forests)    
- [First Steps With PySpark and Big Data Processing](https://realpython.com/pyspark-intro/)  
- [A Brief Introduction to PySpark](https://towardsdatascience.com/a-brief-introduction-to-pyspark-ff4284701873)  
- [Introduction to Anomaly Detection in Python](https://blog.floydhub.com/introduction-to-anomaly-detection-in-python/)  
- [Time Series basics Exploring](https://www.kaggle.com/jagangupta/time-series-basics-exploring-traditional-ts)  
- [Understanding Variational Autoencoders (VAEs)](https://towardsdatascience.com/understanding-variational-autoencoders-vaes-f70510919f73)  
- [Variational Autoencoders](https://www.jeremyjordan.me/variational-autoencoders/)  
- [Benchmarking Performance and Scaling of Python Clustering Algorithms](https://hdbscan.readthedocs.io/en/latest/performance_and_scalability.html) [_Important_]  
- [Comparing Python Clustering Algorithms](https://hdbscan.readthedocs.io/en/latest/comparing_clustering_algorithms.html)  
- [Modular Active Learning framework](https://github.com/modAL-python/modAL)   
- [pycaret: An open-source, low-code machine learning library in Python](https://github.com/pycaret/pycaret) [Good]  
- [Lazy Predict](https://github.com/shankarpandala/lazypredict) [Good]  
- [AutoXGB: XGBoost + Optuna](https://github.com/abhishekkrthakur/autoxgb) [Good]    
- [shap-hypetune: Hyperparameters Tuning and Features Selection for Gradient Boosting Models](https://github.com/cerlymarco/shap-hypetune)  
- [Deepnote: Great data science notebook](https://deepnote.com/)   
- [khanacademy statistics course](https://www.khanacademy.org/math/ap-statistics) [_Good_]   
- [NumPy Exercises](https://www.w3resource.com/python-exercises/numpy/index.php)  [_Good_]  
- [Data Engineering Zoomcamp](https://github.com/DataTalksClub/data-engineering-zoomcamp)  
- [Vaex: high performance Python library for DataFrames](https://github.com/vaexio/vaex) [Great]  

## The best exploratory data analysis (EDA) Tools:
- [pandas-profiling](https://github.com/pandas-profiling/pandas-profiling)  
- [sweetviz](https://github.com/fbdesignpro/sweetviz)  
- [DataPrep: The easiest way to prepare data in Python](https://dataprep.ai/)   
- [Speeding Up Exploratory Data Analysis with Python](https://towardsdatascience.com/speeding-up-exploratory-data-analysis-with-python-838fe5e25b43)   

## Scikit-learn Algorithms on GPU & for Large-Scale Data Sets:
- [skorch - scikit-learn compatible neural network library that wraps PyTorch](https://github.com/skorch-dev/skorch)  
- [scikit-cuda](https://github.com/lebedov/scikit-cuda)  
- [Hummingbird - trained traditional ML models into tensor computations](https://github.com/microsoft/hummingbird)  

## Anomaly Detection:
- [An Awesome Tutorial to Learn Outlier Detection in Python using PyOD Library](https://www.analyticsvidhya.com/blog/2019/02/outlier-detection-python-pyod/)  
- [PyOD: Python Outlier Detection](https://github.com/yzhao062/pyod) [**Great**]  
- [PyNomaly](https://github.com/vc1492a/PyNomaly)  
- [DeepLog: Anomaly detection and diagnosis from system logs through deep learning](https://github.com/Thijsvanede/DeepLog)  

## Bayesian Neural Networks (BNNs) & Bayesian Recommendar System:  
- [TyXe: Pyro-based BNNs for Pytorch users](https://github.com/TyXe-BDL/TyXe)   
- [Bayesian Personalized Ranking](https://github.com/shah314/BPR)   
- [PYROS: PYthon RecOmmender Systems library](https://github.com/makgyver/pyros)   
- [One-class Bayesian Personalized Ranking](https://github.com/YifanZhou95/Bayesian-Personalized-Ranking)   

## Modeling Numerical & Categorical Data in ML:
- [How to Prepare Categorical Data for Deep Learning in Python](https://machinelearningmastery.com/how-to-prepare-categorical-data-for-deep-learning-in-python/)  
- [Handling Categorical Data in ML Models](https://www.pluralsight.com/guides/handling-categorical-data-in-machine-learning-models)   
- [Encoding Categorical Data](https://www.analyticsvidhya.com/blog/2020/08/types-of-categorical-data-encoding/)  
## Feature Engineering:
- [Lime: Explaining the predictions of any machine learning classifier](https://github.com/marcotcr/lime)  
- [Lime Tutorial: Building Trust in Machine Learning Models (using LIME in Python)](https://www.analyticsvidhya.com/blog/2017/06/building-trust-in-machine-learning-models/)  
- [Missing Values: End-to-End Introduction to Handling Missing Values](https://www.analyticsvidhya.com/blog/2021/10/end-to-end-introduction-to-handling-missing-values/)    

## Click-Through Rate (CTR) Prediction:
- [DeepCTR](https://github.com/shenweichen/DeepCTR)   

## Dynamic Time Warping (Time Series):
- [Dynamic Time Warping](https://en.wikipedia.org/wiki/Dynamic_time_warping)  

## Data Science Courses:
- [Data Science Course](https://github.com/briandalessandro/DataScienceCourse)  

## Working with Tabular Data:
- [Yandex RTDL Library](https://github.com/yandex-research/rtdl)   
- [Vaex: high performance Python library for DataFrames](https://github.com/vaexio/vaex) [Great]  

## Data Science Interview Notes:
- [Cracking the Data Science Interview](https://github.com/khanhnamle1994/cracking-the-data-science-interview)  
- [Data Engineer Interview Questions Python](https://realpython.com/data-engineer-interview-questions-python/)  
- [Facebook Data Scientist Interview Questions](https://www.glassdoor.com/Interview/Facebook-Data-Scientist-Interview-Questions-EI_IE40772.0,8_KO9,23.htm)   
- [Interview Guides: Facebook Data Scientist](https://prepfully.com/interview-guides/facebook-data-scientist)   
- [Meta (Facebook) Data Science Interview Questions and Solutions](https://www.interviewquery.com/blog-facebook-data-science-interview-questions-and-solutions)   

## Deep Learning Recommendation Model:
- [Introducing TorchRec](https://pytorch.org/blog/introducing-torchrec/)  
- [TorchRec](https://github.com/pytorch/torchrec)  
- [Deep Learning Recommendation Model for Personalization and Recommendation Systems - DLRM](https://github.com/facebookresearch/dlrm)  
- [DLRM: An advanced, open source deep learning recommendation model](https://ai.facebook.com/blog/dlrm-an-advanced-open-source-deep-learning-recommendation-model/)  
- [LightFM](https://github.com/lyst/lightfm)  
- [Neural Recommendation Algorithms](https://towardsdatascience.com/recotour-ii-neural-recommendation-algorithms-49733938d56e)  
- [Build a Recommendation Engine With Collaborative Filtering](https://realpython.com/build-recommendation-engine-collaborative-filtering/)  [_**Great**_]  
- [NCF - Neural Collaborative Filtering](https://github.com/NervanaSystems/distiller/tree/master/examples/ncf)  
- [Using Neural Networks for your Recommender System](https://developer.nvidia.com/blog/using-neural-networks-for-your-recommender-system/) [**Great**]     
- [Neural Collaborative Filtering](https://towardsdatascience.com/neural-collaborative-filtering-96cef1009401)  
- [AWS Personalized Recommendation Model](https://aws.amazon.com/personalize/)  
- [Microsoft Recommenders](https://github.com/microsoft/recommenders) [Great]  
- [Monolith: Real Time Recommendation System of TikTok](https://arxiv.org/pdf/2209.07663.pdf)  
- [Twitter's Recommendation Algorithm](https://github.com/twitter/the-algorithm-ml)  
- [TikTok recommender system Notes](https://www.linkedin.com/posts/damienbenveniste_machinelearning-datascience-artificialintelligence-activity-7050129099549249536-TctQ/?utm_source=share&utm_medium=member_android)  
